<?php
namespace Drupal\node_access_helper;

use Drupal\node\Entity\Node;
use Drupal\node\NodeInterface;

class NodeGrantsUpdater {

  public static function update($node_data): void {
    // Do the obvious simple one fastest and first.
    if ($node_data instanceof NodeInterface) {
      self::updateNode($node_data);
      return;
    }
    // Accept people like me throwing anything at this.

    // Convert node ID into node object.
    if (is_numeric($node_data)) {
      $nid = (int) $node_data;
      $node = Node::load($nid);
      if ($node) {
        self::updateNode($node);
      }
      return;
    }

    // If we have an array, split it up and try this method again.
    if (is_array($node_data)) {
      self::updateMultiple($node_data);
      return;
    }
  }

  public static function updateMultiple($node_data): void {
    foreach ($node_data as $node_datum) {
      self::update($node_datum);
    }
  }

  public static function updateNode(NodeInterface $node): void {
    // This is the same approach used by Drupal core's node_access_rebuild, see
    // https://api.drupal.org/api/drupal/core%21modules%21node%21node.module/function/node_access_rebuild/11.x
    /** @var \Drupal\node\NodeAccessControlHandlerInterface $access_control_handler */
    $access_control_handler = \Drupal::entityTypeManager()->getAccessControlHandler('node');
    $grants = $access_control_handler->acquireGrants($node);
    \Drupal::service('node.grant_storage')->write($node, $grants); 
  }
}